import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import localeDeExtra from '@angular/common/locales/extra/de';
import { SnackBarE2EComponent } from 'apps/alfa-e2e/src/components/ui/snackbar.e2e.component';
import { VorgangZusammenarbeitE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-zusammenarbeit.e2e.component';
import { VorgangE2E, VorgangMessagesE2E, VorgangStatusE2E } from 'apps/alfa-e2e/src/model/vorgang';
import 'cypress-real-events/support';
import { VorgangListE2EComponent } from '../../../components/vorgang/vorgang-list.e2e.component';
import { MainPage, waitForSpinnerToDisappear } from '../../../page-objects/main.po';
import { VorgangPage } from '../../../page-objects/vorgang.po';
import { dropCollections } from '../../../support/cypress-helper';
import { contains, exist, notExist } from '../../../support/cypress.util';
import { initUsermanagerUsers, loginAsSabine } from '../../../support/user-util';
import { buildVorgang, initVorgaenge, objectIds } from '../../../support/vorgang-util';

registerLocaleData(localeDe, 'de', localeDeExtra);

describe('Vorgang Zusammenarbeit anfragen', () => {
  const mainPage: MainPage = new MainPage();
  const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();

  const vorgangPage: VorgangPage = new VorgangPage();
  const zusammenarbeitContainer: VorgangZusammenarbeitE2EComponent =
    vorgangPage.getZusammenarbeitContainer();

  const zusammenarbeitVorgang: VorgangE2E = {
    ...buildVorgang(objectIds[0], 'Zusammenarbeit Vorgang'),
    status: VorgangStatusE2E.IN_BEARBEITUNG,
  };

  const titleText: string = 'Dies ist ein Test-Titel !"§$%&';
  const messageText: string =
    'Sehr geehrter Tester\n\n Dies ist ein !"§$%& Test\n zum Testen der Nachricht.\n\n\n\nhier sollte eine \nScrollbar\nangezeigt\nwerden!\n\nMfG!';
  const stelleSearch1: string = 'k';
  const stelleSearch2: string = 'Kiel';
  const snackBar: SnackBarE2EComponent = mainPage.getSnackBar();

  before(() => {
    initVorgaenge([zusammenarbeitVorgang]);
    initUsermanagerUsers();

    loginAsSabine();

    waitForSpinnerToDisappear();
    exist(vorgangList.getRoot());
  });

  after(() => {
    dropCollections();
  });

  describe('create new Anfrage', () => {
    it('should show button for Zusammenarbeit', () => {
      vorgangList.getListItem(zusammenarbeitVorgang.name).getRoot().click();
      waitForSpinnerToDisappear();

      exist(zusammenarbeitContainer.getAnfrageButton());
    });

    it('should show input elements on Anfrage click', () => {
      zusammenarbeitContainer.createAnfrage();

      exist(zusammenarbeitContainer.getZustaendigeStelleButton());
      exist(zusammenarbeitContainer.getStelleTitel());
      exist(zusammenarbeitContainer.getMessageText());
      exist(zusammenarbeitContainer.getSendButton());
      exist(zusammenarbeitContainer.getCancelButton());
    });

    it('should close elements on Cancel click', () => {
      zusammenarbeitContainer.cancelAnfrage();

      notExist(zusammenarbeitContainer.getZustaendigeStelleButton());
      notExist(zusammenarbeitContainer.getStelleTitel());
      notExist(zusammenarbeitContainer.getMessageText());
      notExist(zusammenarbeitContainer.getSendButton());
      notExist(zusammenarbeitContainer.getCancelButton());
      exist(zusammenarbeitContainer.getAnfrageButton());
    });

    it('should open new search label for Zustaendige Stelle', () => {
      zusammenarbeitContainer.createAnfrage();
      zusammenarbeitContainer.searchZustaendigeStelle();

      exist(zusammenarbeitContainer.getCloseSearchButton());
      exist(zusammenarbeitContainer.getSearchText());
    });

    it('should close layer on Cancel click', () => {
      zusammenarbeitContainer.closeSearch();

      notExist(zusammenarbeitContainer.getCloseSearchButton());
      notExist(zusammenarbeitContainer.getSearchText());
      exist(zusammenarbeitContainer.getZustaendigeStelleButton());
    });

    it('should delete search term on clicking X', () => {
      zusammenarbeitContainer.searchZustaendigeStelle();
      zusammenarbeitContainer.enterSearchInput(stelleSearch1);
      zusammenarbeitContainer.clearSearch();

      zusammenarbeitContainer.hasSearchText('');
    });

    it.skip('TODO: Setup Jenkins --- should find 2 results after entering kiel', () => {
      zusammenarbeitContainer.enterSearchInput(stelleSearch2);

      zusammenarbeitContainer.expectNumberOfEntriesToBe(2);
    });

    it.skip('should close layer after click on search entry', () => {
      zusammenarbeitContainer.clickSearchEntry(0);

      exist(zusammenarbeitContainer.getSendButton());
      notExist(zusammenarbeitContainer.getSearchText());
    });

    it.skip('should show part of address in Zufi header', () => {
      zusammenarbeitContainer.addressContains(stelleSearch2);
    });

    it.skip('should be able to enter title and message, and show scrollbar on long text', () => {
      zusammenarbeitContainer.enterTitel(titleText);
      zusammenarbeitContainer.enterMessage(messageText);

      zusammenarbeitContainer.messageScrollbarIsPresent();
    });

    it.skip('should show title and message, show snackbar, and remove buttons after sending', () => {
      zusammenarbeitContainer.sendAnfrage();

      notExist(zusammenarbeitContainer.getSendButton());
      notExist(zusammenarbeitContainer.getCancelButton());

      zusammenarbeitContainer.anfrageResultContains(titleText);

      //TODO: Zeilenumbrüche (OZG-6682)
      //zusammenarbeitContainer.anfrageResultContains(messageText);

      //TODO: Datum wird angezeigt (OZG-6675)

      contains(snackBar.getMessage(), VorgangMessagesE2E.ZUARBEIT_ANGEFRAGT);
    });
  });
});
