import { Environment, ENVIRONMENT_CONFIG } from '@alfa-client/environment-shared';
import { Inject, Injectable } from '@angular/core';
import { AuthConfig, OAuthService } from 'angular-oauth2-oidc';
import { JwksValidationHandler } from 'angular-oauth2-oidc-jwks';
import { UserProfileResource } from 'libs/user-profile-shared/src/lib/user-profile.model';
import { getUserNameInitials } from 'libs/user-profile-shared/src/lib/user-profile.util';

@Injectable({ providedIn: 'root' })
export class AuthenticationService {
  currentUserResource: UserProfileResource;

  constructor(
    private oAuthService: OAuthService,
    @Inject(ENVIRONMENT_CONFIG) private envConfig: Environment,
  ) {}

  public async login(): Promise<void> {
    this.oAuthService.configure(this.buildConfiguration());
    this.oAuthService.setupAutomaticSilentRefresh();
    this.oAuthService.tokenValidationHandler = new JwksValidationHandler();
    await this.oAuthService.loadDiscoveryDocumentAndLogin();
    this.setCurrentUser();
  }

  buildConfiguration(): AuthConfig {
    return {
      issuer: this.envConfig.authServer + '/realms/' + this.envConfig.realm,
      tokenEndpoint:
        this.envConfig.authServer +
        '/realms/' +
        this.envConfig.realm +
        '/protocol/openid-connect/token',
      redirectUri: window.location.origin + '/',
      clientId: this.envConfig.clientId,
      scope: 'openid profile',
      requireHttps: false,
      responseType: 'code',
      showDebugInformation: false,
    };
  }

  setCurrentUser(): void {
    const claims: Record<string, any> = this.oAuthService.getIdentityClaims();
    const userResource: UserProfileResource = <any>{
      firstName: claims['given_name'],
      lastName: claims['family_name'],
    };
    this.currentUserResource = userResource;
  }

  public getCurrentUserInitials(): string {
    return getUserNameInitials(this.currentUserResource);
  }

  public logout(): void {
    this.oAuthService.revokeTokenAndLogout();
  }
}
