import {
  OrganisationsEinheitResource,
  OrganisationsEinheitService,
} from '@alfa-client/collaboration-shared';
import { isNotNull } from '@alfa-client/tech-shared';
import { OzgcloudDialogService } from '@alfa-client/ui';
import { DialogConfig } from '@angular/cdk/dialog';
import { Component, Input, OnInit, ViewContainerRef } from '@angular/core';
import { AbstractControl } from '@angular/forms';
import { ResourceUri, getUrl } from '@ngxp/rest';
import { Observable, tap } from 'rxjs';
import { SearchOrganisationsEinheitContainerComponent } from '../../../search-organisations-einheit-container/search-organisations-einheit-container.component';

const DIALOG_CONFIG: DialogConfig = {
  backdropClass: ['backdrop-blur-1', 'bg-greybackdrop'],
};

@Component({
  selector: 'alfa-organisations-einheit-container',
  templateUrl: './organisations-einheit-container.component.html',
  providers: [OrganisationsEinheitService],
})
export class OrganisationsEinheitContainerComponent implements OnInit {
  @Input() public fieldControl: AbstractControl<ResourceUri>;

  public organisationsEinheitResource$: Observable<OrganisationsEinheitResource>;

  constructor(
    private readonly dialogService: OzgcloudDialogService,
    readonly viewContainerRef: ViewContainerRef,
    private readonly service: OrganisationsEinheitService,
  ) {}

  ngOnInit(): void {
    this.organisationsEinheitResource$ = this.getSelectedResult();
  }

  getSelectedResult(): Observable<OrganisationsEinheitResource> {
    return this.service
      .getSelectedResult()
      .pipe(
        tap((organisationsEinheitResource: OrganisationsEinheitResource) =>
          this.handleResult(organisationsEinheitResource),
        ),
      );
  }

  handleResult(organisationsEinheitResource: OrganisationsEinheitResource): void {
    if (isNotNull(organisationsEinheitResource)) {
      this.fieldControl.patchValue(getUrl(organisationsEinheitResource));
    }
  }

  public openSearchDialog(): void {
    this.dialogService.openInCallingComponentContext<SearchOrganisationsEinheitContainerComponent>(
      SearchOrganisationsEinheitContainerComponent,
      this.viewContainerRef,
      null, //FIXME bitte null nicht als Parameter nehmen
      DIALOG_CONFIG,
    );
  }
}
