import { TechSharedModule, isNotNil } from '@alfa-client/tech-shared';
import { Component, EventEmitter, HostListener, Input, Output } from '@angular/core';
import { ReactiveFormsModule } from '@angular/forms';
import { FileUploadButtonComponent, SpinnerIconComponent } from '@ods/system';
import { uniqueId } from 'lodash-es';
import { FormControlEditorAbstractComponent } from '../formcontrol-editor.abstract.component';

@Component({
  selector: 'ods-single-file-upload-editor',
  templateUrl: './single-file-upload-editor.component.html',
  standalone: true,
  imports: [FileUploadButtonComponent, SpinnerIconComponent, ReactiveFormsModule, TechSharedModule],
})
export class SingleFileUploadEditorComponent extends FormControlEditorAbstractComponent {
  @Input() label: string = '';
  @Input() accept: string = '*/*';
  @Input() uploadInProgress: boolean;

  @Output() public newFile: EventEmitter<File> = new EventEmitter<File>();

  public readonly uploadFileId: string = uniqueId();

  @HostListener('change', ['$event.target.files']) emitFiles(event: FileList): void {
    if (isNotNil(event)) {
      this.newFile.emit(event.item(0));
      this.setErrors();
    }
  }
}
