import { type Meta, type StoryObj } from '@storybook/angular';
import { AttachmentComponent } from './attachment.component';

const meta: Meta<AttachmentComponent> = {
  title: 'Attachment',
  component: AttachmentComponent,
  excludeStories: /.*Data$/,
  tags: ['autodocs'],
};

export default meta;
type Story = StoryObj<AttachmentComponent>;

export const Default: Story = {
  name: 'File with pdf icon',
  args: {
    caption: 'Katzenanmeldung_1231231_eingang.pdf',
    errorCaption: 'Error!',
    loadingCaption: 'Loading...',
    description: '450 kB',
    fileType: 'pdf',
    isLoading: false,
    errorMessages: [],
  },
  argTypes: {
    caption: {
      description: 'Name of the document',
    },
    errorCaption: {
      description: 'Error text appears when eror messages array is not empty',
    },
    loadingCaption: {
      description: 'Loading text appears when isLoading flag is true',
    },
    description: {
      description: 'Description can contain size of file or e.g. loading progress',
    },
    fileType: {
      description: 'Type of file icon',
    },
    isLoading: {
      description: 'Boolean flag for loading state',
      table: {
        defaultValue: { summary: 'false' },
      },
    },
    errorMessages: {
      description: 'The array containing strings with errors text',
    },
  },
};

export const Doc: Story = {
  name: 'File with doc icon',
  args: {
    caption: 'Katzenanmeldung_1231231_eingang.doc',
    description: '573 kB',
    fileType: 'doc',
  },
};

export const Loading: Story = {
  name: 'File with loading icon',
  args: {
    loadingCaption: 'Katzenanmeldung_1231231_eingang.doc',
    description: '573 kB',
    fileType: 'doc',
    isLoading: true,
  },
};

export const Error: Story = {
  name: 'Error message in file',
  args: {
    errorCaption: 'Fehler beim Hochladen',
    errorMessages: ['Erlaubte Dateiendungen: pdf, jpg, png, jpeg'],
  },
};
