import {
  argsToTemplate,
  componentWrapperDecorator,
  moduleMetadata,
  type Meta,
  type StoryObj,
} from '@storybook/angular';

import { SaveIconComponent } from '../../icons/save-icon/save-icon.component';
import { UserIconComponent } from '../../icons/user-icon/user-icon.component';
import { PopupListItemComponent } from '../popup-list-item/popup-list-item.component';
import { PopupComponent } from './popup.component';

const meta: Meta<PopupComponent> = {
  title: 'Popup/Popup',
  component: PopupComponent,
  decorators: [
    moduleMetadata({
      imports: [PopupComponent, PopupListItemComponent, SaveIconComponent, UserIconComponent],
    }),
    componentWrapperDecorator((story) => `<div class="flex justify-center mb-32">${story}</div>`),
  ],
  excludeStories: /.*Data$/,
  tags: ['autodocs'],
};

export default meta;
type Story = StoryObj<PopupComponent>;

export const Default: Story = {
  args: { alignTo: 'left', label: '', buttonClass: '' },
  argTypes: {
    alignTo: {
      control: 'select',
      options: ['left', 'right'],
      table: {
        defaultValue: { summary: 'left' },
      },
    },
    buttonClass: { description: 'Tailwind class for button' },
    label: { description: 'Aria-label for button' },
  },
  render: (args) => ({
    props: args,
    template: `<ods-popup ${argsToTemplate(args)}>
        <ods-user-icon button-content />
        <ods-popup-list-item caption="Lorem" />
        <ods-popup-list-item caption="Ipsum" />
        <ods-popup-list-item caption="Dolor" />
      </ods-popup>`,
  }),
};

export const LongText: Story = {
  render: (args) => ({
    props: args,
    template: `<ods-popup ${argsToTemplate(args)}>
        <p button-content>Trigger popup</p>
        <ods-popup-list-item caption="Lorem" />
        <ods-popup-list-item caption="Lorem ipsum dolor sit amet" />
      </ods-popup>`,
  }),
};

export const ItemsWithIcons: Story = {
  render: (args) => ({
    props: args,
    template: `<ods-popup ${argsToTemplate(args)}>
        <p button-content>Trigger popup</p>
        <ods-popup-list-item caption="Lorem">
          <ods-save-icon icon size="small" />
        </ods-popup-list-item>
        <ods-popup-list-item caption="Lorem ipsum dolor sit amet">
          <ods-save-icon icon size="small" />
        </ods-popup-list-item>
      </ods-popup>`,
  }),
};
