/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { KommentarListLinkRel, KommentarService } from '@alfa-client/kommentar-shared';
import { createStateResource } from '@alfa-client/tech-shared';
import { mock } from '@alfa-client/test-utils';
import {
  ExpansionPanelComponent,
  OzgcloudStrokedButtonWithSpinnerComponent,
} from '@alfa-client/ui';
import { cold } from 'jest-marbles';
import { MockComponent } from 'ng-mocks';
import { Observable } from 'rxjs';
import { createKommentarListResource } from '../../../../kommentar-shared/test/kommentar';
import { createVorgangWithEingangResource } from '../../../../vorgang-shared/test/vorgang';
import { KommentarFormComponent } from './kommentar-form/kommentar-form.component';
import { KommentarListInVorgangContainerComponent } from './kommentar-list-in-vorgang-container.component';
import { KommentarListInVorgangComponent } from './kommentar-list-in-vorgang/kommentar-list-in-vorgang.component';

describe('KommentarListInVorgangContainerComponent', () => {
  let component: KommentarListInVorgangContainerComponent;
  let fixture: ComponentFixture<KommentarListInVorgangContainerComponent>;

  const kommentarService = mock(KommentarService);

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        KommentarListInVorgangContainerComponent,
        MockComponent(ExpansionPanelComponent),
        MockComponent(KommentarListInVorgangComponent),
        MockComponent(KommentarFormComponent),
        MockComponent(OzgcloudStrokedButtonWithSpinnerComponent),
      ],
      providers: [
        {
          provide: KommentarService,
          useValue: kommentarService,
        },
      ],
    });
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(KommentarListInVorgangContainerComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
    component.vorgangStateResource = createStateResource(createVorgangWithEingangResource());
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('ng on changes', () => {
    beforeEach(() => {
      kommentarService.isFormularVisible.mockReturnValue(new Observable((o) => o.next(false)));
      kommentarService.getKommentareByVorgang.mockReturnValue(
        new Observable((o) =>
          o.next(
            createStateResource(
              createKommentarListResource([KommentarListLinkRel.CREATE_KOMMENTAR]),
            ),
          ),
        ),
      );
    });

    it('should call kommentar service isFormularVisible', () => {
      component.ngOnChanges();

      expect(kommentarService.isFormularVisible).toHaveBeenCalled();
    });

    it('should call kommentar service getKommentareByVorgang', () => {
      component.ngOnChanges();

      expect(kommentarService.getKommentareByVorgang).toHaveBeenCalledWith(
        component.vorgangStateResource.resource,
      );
    });

    it('should create new Kommentare', () => {
      const canCreateNewKommentar$ = cold('a', { a: true });
      const kommentarListStateResource = createStateResource(createKommentarListResource());
      const kommentarListStateResource$ = new Observable((observer) =>
        observer.next(kommentarListStateResource),
      );
      kommentarService.getKommentareByVorgang.mockReturnValue(kommentarListStateResource$);
      kommentarService.canCreateNewKommentar.mockReturnValue(
        new Observable((observer) => observer.next(true)),
      );

      component.ngOnChanges();

      expect(component.canCreateNewKommentar$).toBeObservable(canCreateNewKommentar$);
    });

    it('should not create new Kommentare', () => {
      const canCreateNewKommentar$ = cold('a', { a: false });
      const kommentarListStateResource = createStateResource(createKommentarListResource());
      const kommentarListStateResource$ = new Observable((observer) =>
        observer.next(kommentarListStateResource),
      );
      kommentarService.getKommentareByVorgang.mockReturnValue(kommentarListStateResource$);
      kommentarService.canCreateNewKommentar.mockReturnValue(
        new Observable((observer) => observer.next(false)),
      );

      component.ngOnChanges();

      expect(component.canCreateNewKommentar$).toBeObservable(canCreateNewKommentar$);
    });

    it('should call reloadKommentarListOnVorgangReload', () => {
      const reloadKommentarListOnVorgangReload = jest
        .spyOn(component, 'reloadKommentarListOnVorgangReload')
        .mockReturnThis();

      component.ngOnChanges();

      expect(reloadKommentarListOnVorgangReload).toHaveBeenCalled();
    });
  });

  describe('reloadKommentarListOnVorgangReload', () => {
    beforeEach(() => {
      kommentarService.setKommentarListReload.mockClear();
    });

    it('should call kommentarService', () => {
      component.vorgangStateResource = {
        ...createStateResource(createVorgangWithEingangResource()),
        reload: true,
      };

      component.reloadKommentarListOnVorgangReload();

      expect(kommentarService.setKommentarListReload).toBeCalled();
    });

    it('should not call kommentarService', () => {
      component.vorgangStateResource = {
        ...createStateResource(createVorgangWithEingangResource()),
        reload: false,
      };

      component.reloadKommentarListOnVorgangReload();

      expect(kommentarService.setKommentarListReload).not.toBeCalled();
    });
  });

  describe('show formular', () => {
    it('should call kommentar service showFormular', () => {
      component.showFormular();

      expect(kommentarService.showFormular).toHaveBeenCalled();
    });
  });

  describe('hide formular', () => {
    it('should call kommentar service hideFormular', () => {
      component.hideFormular();

      expect(kommentarService.hideFormular).toHaveBeenCalled();
    });
  });
});
