/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { mock, useFromMock } from '@alfa-client/test-utils';
import { VorgangHeaderLinkRel, VorgangResource } from '@alfa-client/vorgang-shared';
import { ResourceFactory } from '@ngxp/rest';
import { cold, hot } from 'jest-marbles';
import { createPostfachMailListResource } from 'libs/postfach-shared/test/postfach';
import { createVorgangResource } from 'libs/vorgang-shared/test/vorgang';
import { PostfachMailListLinkRel } from './postfach.linkrel';
import { PostfachMailListResource } from './postfach.model';
import { PostfachRepository } from './postfach.repository';

describe('PostfachRepository', () => {
  let repository: PostfachRepository;
  const resourceFactory = mock(ResourceFactory);
  const resourceWrapper = { get: jest.fn(), put: jest.fn() };

  beforeEach(() => {
    resourceFactory.from.mockReturnValue(resourceWrapper);

    repository = new PostfachRepository(useFromMock(resourceFactory));
  });

  describe('loadPostfachMailList', () => {
    const postfachMailListResource: PostfachMailListResource = createPostfachMailListResource();
    const vorgang: VorgangResource = createVorgangResource();

    beforeEach(() => {
      resourceWrapper.get.mockReturnValue(hot('a', { a: postfachMailListResource }));
    });

    it('should call resourceFactory with resource', () => {
      repository.loadPostfachMailList(vorgang);

      expect(resourceFactory.from).toHaveBeenCalledWith(vorgang);
    });

    it('should call resourceWrapper with link', () => {
      repository.loadPostfachMailList(vorgang);

      expect(resourceWrapper.get).toHaveBeenCalledWith(VorgangHeaderLinkRel.POSTFACH_MAILS);
    });

    it('should return result', () => {
      const result = repository.loadPostfachMailList(vorgang);

      expect(result).not.toBeNull();
      expect(result).toBeObservable(cold('a', { a: postfachMailListResource }));
    });
  });

  describe('resetHasNewPostfachNachrichten', () => {
    const postfachMailListResource: PostfachMailListResource = createPostfachMailListResource([
      PostfachMailListLinkRel.RESET_HAS_NEW_POSTFACH_NACHRICHT,
    ]);

    it('should call resourceFactory with resource', () => {
      repository.resetHasNewPostfachNachrichten(postfachMailListResource);

      expect(resourceFactory.from).toHaveBeenCalledWith(postfachMailListResource);
    });

    it('should call resourceWrapper with link', () => {
      repository.resetHasNewPostfachNachrichten(postfachMailListResource);

      expect(resourceWrapper.put).toHaveBeenCalledWith(
        PostfachMailListLinkRel.RESET_HAS_NEW_POSTFACH_NACHRICHT,
        { hasNewPostfachNachricht: false },
      );
    });
  });
});
