/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { PostfachService } from '@alfa-client/postfach-shared';
import { Mock, mock } from '@alfa-client/test-utils';
import { OzgcloudStrokedButtonWithSpinnerComponent } from '@alfa-client/ui';
import { MockComponent } from 'ng-mocks';
import { PostfachMailPdfButtonContainerComponent } from './postfach-mail-pdf-button-container.component';
import { PostfachMailPdfButtonComponent } from './postfach-mail-pdf-button/postfach-mail-pdf-button.component';

describe('PostfachMailPdfButtonContainerComponent', () => {
  let component: PostfachMailPdfButtonContainerComponent;
  let fixture: ComponentFixture<PostfachMailPdfButtonContainerComponent>;

  const postfachService: Mock<PostfachService> = mock(PostfachService);

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        PostfachMailPdfButtonContainerComponent,
        MockComponent(OzgcloudStrokedButtonWithSpinnerComponent),
        MockComponent(PostfachMailPdfButtonComponent),
      ],
      providers: [
        {
          provide: PostfachService,
          useValue: postfachService,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(PostfachMailPdfButtonContainerComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('ngOnInit', () => {
    it('should call postfachService', () => {
      component.ngOnInit();

      expect(postfachService.isDownloadPdfInProgress).toHaveBeenCalled();
    });
  });

  describe('downloadPdf', () => {
    it('should call service', () => {
      component.downloadPdf();

      expect(postfachService.downloadPdf).toHaveBeenCalled();
    });
  });
});
