/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { Mock, mock, useFromMock } from '@alfa-client/test-utils';
import { HttpErrorHandler } from '../error/error.handler';
import {
  disableInterceptorDefaultHandling,
  enableInterceptorDefaultHandling,
} from './error.decorator.util';

describe('Error decorator util', () => {
  let httpErrorHandler: Mock<HttpErrorHandler>;

  beforeEach(() => {
    httpErrorHandler = mock(HttpErrorHandler);
  });

  describe('enableInterceptorDefaultHandling', () => {
    it('should enable default error handling', () => {
      enableInterceptorDefaultHandling(useFromMock(httpErrorHandler));

      expect(httpErrorHandler.enableDefaultHandling).toHaveBeenCalled();
    });

    it('should enable default retry handling', () => {
      enableInterceptorDefaultHandling(useFromMock(httpErrorHandler));

      expect(httpErrorHandler.enableRetry).toHaveBeenCalled();
    });
  });

  describe('disableInterceptorDefaultHandling', () => {
    it('should disable default error handling', () => {
      disableInterceptorDefaultHandling(useFromMock(httpErrorHandler));

      expect(httpErrorHandler.disableDefaultHandling).toHaveBeenCalled();
    });

    it('should disable default retry handling', () => {
      disableInterceptorDefaultHandling(useFromMock(httpErrorHandler));

      expect(httpErrorHandler.disableRetry).toHaveBeenCalled();
    });
  });
});
