import { createEmptyStateResource } from '@alfa-client/tech-shared';
import { getElementFromFixture } from '@alfa-client/test-utils';
import { SpinnerComponent } from '@alfa-client/ui';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MatIcon } from '@angular/material/icon';
import { MatProgressSpinner } from '@angular/material/progress-spinner';
import { getDataTestClassOf } from 'libs/tech-shared/test/data-test';
import { SpinnerTransparencyComponent } from './spinner-transparency.component';

describe('SpinnerTransparencyComponent', () => {
  let component: SpinnerTransparencyComponent;
  let fixture: ComponentFixture<SpinnerTransparencyComponent>;

  const spinnerSelector: string = getDataTestClassOf('spinner-transparency');

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [SpinnerTransparencyComponent, SpinnerComponent],
      imports: [MatIcon, MatProgressSpinner],
    }).compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(SpinnerTransparencyComponent);
    component = fixture.componentInstance;

    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('spinner', () => {
    it('should NOT be visible if NOT loading and already loaded', () => {
      component.stateResource = { ...createEmptyStateResource(false), loaded: true };
      fixture.detectChanges();

      const spinner = getElementFromFixture(fixture, spinnerSelector);

      expect(spinner).not.toBeInstanceOf(HTMLElement);
    });

    it('should be visible if loading', () => {
      component.stateResource = createEmptyStateResource(true);
      fixture.detectChanges();

      const spinner = getElementFromFixture(fixture, spinnerSelector);

      expect(spinner).toBeInstanceOf(HTMLElement);
    });
  });
});
