import { BescheidService } from '@alfa-client/bescheid-shared';
import { CommandResource, tapOnCommandSuccessfullyDone } from '@alfa-client/command-shared';
import { StateResource } from '@alfa-client/tech-shared';
import { OzgcloudDialogService } from '@alfa-client/ui';
import { Component } from '@angular/core';
import { Observable } from 'rxjs';
import { BescheidenFormService } from '../../../bescheiden.formservice';

@Component({
  selector: 'alfa-vorgang-detail-bescheiden-abbrechen-dialog',
  templateUrl: './vorgang-detail-bescheiden-abbrechen-dialog.component.html',
})
export class VorgangDetailBescheidenAbbrechenDialogComponent {
  saveDraftInProgress$: Observable<StateResource<CommandResource>>;
  deleteBescheid$: Observable<StateResource<CommandResource>>;

  constructor(
    private readonly bescheidService: BescheidService,
    private readonly ozgcloudDialogService: OzgcloudDialogService,
    private readonly formService: BescheidenFormService,
  ) {}

  public saveBescheidDraft() {
    this.saveDraftInProgress$ = this.formService.submitDraft().pipe(
      tapOnCommandSuccessfullyDone(() => {
        this.ozgcloudDialogService.closeAll();
        this.bescheidService.refreshList();
      }),
    );
  }

  public onCancel() {
    if (this.bescheidService.existsBescheidDraft()) {
      this.deleteBescheidDraft();
    } else {
      this.ozgcloudDialogService.closeAll();
    }
  }

  deleteBescheidDraft(): void {
    this.deleteBescheid$ = this.bescheidService.bescheidVerwerfen().pipe(
      tapOnCommandSuccessfullyDone(() => {
        this.ozgcloudDialogService.closeAll();
      }),
    );
  }
}
