import { BescheidResource } from '@alfa-client/bescheid-shared';
import { PostfachService } from '@alfa-client/postfach-shared';
import { OzgcloudDialogService } from '@alfa-client/ui';
import { VorgangService } from '@alfa-client/vorgang-shared';
import { DIALOG_DATA, DialogRef } from '@angular/cdk/dialog';
import { Component, Inject, OnDestroy, OnInit, ViewContainerRef } from '@angular/core';
import { Subscription, filter } from 'rxjs';
import { BescheidenFormService } from './bescheiden.formservice';
import { BescheidenDialogData } from './bescheiden.model';
import { VorgangDetailBescheidenAbbrechenDialogComponent } from './vorgang-detail-bescheiden-steps/vorgang-detail-bescheiden-steps-content/vorgang-detail-bescheiden-abbrechen-dialog/vorgang-detail-bescheiden-abbrechen-dialog.component';

@Component({
  selector: 'alfa-vorgang-detail-bescheiden',
  templateUrl: './vorgang-detail-bescheiden.component.html',
  providers: [BescheidenFormService],
})
export class VorgangDetailBescheidenComponent implements OnDestroy, OnInit {
  private readonly bescheidDraftResource: BescheidResource;

  keydownEventsSubscription: Subscription;

  public activeStep: number = 1;

  constructor(
    public dialogRef: DialogRef,
    public readonly formService: BescheidenFormService,
    private readonly vorgangService: VorgangService,
    private readonly postfachService: PostfachService,
    @Inject(DIALOG_DATA) private readonly dialogData: BescheidenDialogData,
    private readonly ozgcloudDialogService: OzgcloudDialogService,
    readonly viewContainerRef: ViewContainerRef,
  ) {
    this.bescheidDraftResource = dialogData.bescheidDraftResource;
  }

  ngOnInit(): void {
    this.formService.setVorgangWithEingangResource(this.dialogData.vorgangWithEingangResource);
    this.formService.patchValues(this.bescheidDraftResource);
    this.handleEscapeKey();
  }

  handleEscapeKey(): void {
    this.keydownEventsSubscription = this.dialogRef.keydownEvents
      .pipe(filter((event) => event.key === 'Escape'))
      .subscribe(() => this.cancelWizard());
  }

  ngOnDestroy(): void {
    this.keydownEventsSubscription.unsubscribe();
  }

  public onClose(): void {
    this.dialogRef.close();
    this.vorgangService.reloadCurrentVorgang();
    this.postfachService.setPostfachMailOnReload();
  }

  public cancelWizard(): void {
    this.ozgcloudDialogService.openInCallingComponentContext<VorgangDetailBescheidenAbbrechenDialogComponent>(
      VorgangDetailBescheidenAbbrechenDialogComponent,
      this.viewContainerRef,
    );
  }
}
