/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { formatToPrettyDate } from '@alfa-client/tech-shared';
import { VorgangHeaderLinkRel, VorgangResource } from '@alfa-client/vorgang-shared';
import { getAktenzeichenText } from '@alfa-client/vorgang-shared-ui';
import { Component, Input, OnInit } from '@angular/core';
import { hasLink } from '@ngxp/rest';

@Component({
  selector: 'alfa-vorgang-list-item',
  templateUrl: './vorgang-list-item.component.html',
})
export class VorgangListItemComponent implements OnInit {
  @Input() vorgang: VorgangResource;

  readonly vorgangLinkRel = VorgangHeaderLinkRel;

  public ariaLabel: string = '';

  ngOnInit(): void {
    this.buildAriaLabel();
  }

  buildAriaLabel() {
    const name: string = this.vorgang.name;
    const aktenzeichen: string = getAktenzeichenText(this.vorgang);
    const nummer: string = this.vorgang.nummer;
    const status: string = this.vorgang.status;
    const createdAt: string = formatToPrettyDate(this.vorgang.createdAt);

    this.ariaLabel = `Vorgang: ${name}, Aktenzeichen: ${aktenzeichen}, Nummer: ${nummer} Status: ${status}, Eingang: ${createdAt}`;

    if (
      hasLink(this.vorgang, VorgangHeaderLinkRel.WIEDERVORLAGEN) &&
      this.vorgang.nextFrist !== null
    ) {
      this.ariaLabel += this.getWiedervorlageText();
    }

    if (this.vorgang.hasPostfachNachricht) {
      this.ariaLabel += this.getPostfachNachricht();
    }
  }

  getWiedervorlageText(): string {
    const nextFrist: string = formatToPrettyDate(this.vorgang.nextFrist);
    return `, Nächste Wiedervorlage am ${nextFrist}`;
  }

  getPostfachNachricht() {
    if (this.vorgang.hasNewPostfachNachricht) {
      return ', enthält neue Postfachnachrichten';
    }

    return ', enthält Postfachnachrichten';
  }
}
