/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { RouterTestingModule } from '@angular/router/testing';
import { ApiRootLinkRel } from '@alfa-client/api-root-shared';
import {
  HasLinkPipe,
  createEmptyStateResource,
  createStateResource,
} from '@alfa-client/tech-shared';
import { getElementFromFixture } from '@alfa-client/test-utils';
import { createApiRootResource } from 'libs/api-root-shared/test/api-root';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { MockComponent } from 'ng-mocks';
import { VorgangFilterMenuContainerComponent } from './vorgang-filter-menu-container/vorgang-filter-menu-container.component';
import { VorgangListPageComponent } from './vorgang-list-page.component';
import { VorgangViewsMenuContainerComponent } from './vorgang-views-menu-container/vorgang-views-menu-container.component';

describe('VorgangListPageComponent', () => {
  let component: VorgangListPageComponent;
  let fixture: ComponentFixture<VorgangListPageComponent>;

  const userMessage: string = getDataTestIdOf('user-no-role-message');
  const vorgaengeList: string = getDataTestIdOf('vorgaenge-list');

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        VorgangListPageComponent,
        HasLinkPipe,
        MockComponent(VorgangFilterMenuContainerComponent),
        MockComponent(VorgangViewsMenuContainerComponent),
      ],
      imports: [RouterTestingModule],
    }).compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(VorgangListPageComponent);
    component = fixture.componentInstance;
    component.apiRootStateResource = createEmptyStateResource();
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('page', () => {
    it('should show vorgaenge if link exists', () => {
      component.apiRootStateResource = createStateResource(
        createApiRootResource([ApiRootLinkRel.ALLE_VORGAENGE]),
      );
      fixture.detectChanges();

      const element = getElementFromFixture(fixture, vorgaengeList);

      expect(element).toBeInstanceOf(HTMLElement);
    });

    it('should show hint message if link not exists', () => {
      component.apiRootStateResource = createStateResource(createApiRootResource());
      fixture.detectChanges();

      const element = getElementFromFixture(fixture, userMessage);

      expect(element).toBeInstanceOf(HTMLElement);
    });
  });
});
