/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.attachment;

import java.io.OutputStream;

import org.springframework.hateoas.CollectionModel;
import org.springframework.hateoas.EntityModel;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.servlet.mvc.method.annotation.StreamingResponseBody;

import de.ozgcloud.alfa.common.binaryfile.BinaryFileModelAssembler;
import de.ozgcloud.alfa.common.file.OzgFile;
import de.ozgcloud.alfa.common.file.OzgFileService;
import de.ozgcloud.alfa.common.zipdownload.ZipDownloadService;
import de.ozgcloud.alfa.vorgang.VorgangController;
import lombok.RequiredArgsConstructor;

@RestController
@RequestMapping(AttachmentController.PATH)
@RequiredArgsConstructor
public class AttachmentController {

	static final String PATH = "/api/attachments"; // NOSONAR

	static final String PARAM_EINGANG_ID = "eingangId";

	private final OzgFileService fileService;
	private final BinaryFileModelAssembler modelAssembler;

	@GetMapping(params = PARAM_EINGANG_ID)
	public CollectionModel<EntityModel<OzgFile>> getAllByEingang(@RequestParam String eingangId) {
		return modelAssembler.toCollectionModel(fileService.getAttachmentsByEingang(eingangId));
	}

	@RestController
	@RequestMapping(AttachmentsByVorgangController.PATH)
	@RequiredArgsConstructor
	public static class AttachmentsByVorgangController {
		static final String PATH = "/api/vorgang/{vorgangId}/attachments"; // NOSONAR

		private final OzgFileService fileService;
		private final ZipDownloadService zipDownloadService;
		private final VorgangController vorgangController;

		@GetMapping(params = PARAM_EINGANG_ID, produces = MediaType.APPLICATION_OCTET_STREAM_VALUE)
		public ResponseEntity<StreamingResponseBody> download(@PathVariable String vorgangId, @RequestParam String eingangId) {
			return ResponseEntity.ok()
					.header(HttpHeaders.CONTENT_DISPOSITION, String.format("attachment; filename=%s", buildZipName(vorgangId)))
					.contentType(MediaType.APPLICATION_OCTET_STREAM)
					.body(out -> createZipFile(out, eingangId));
		}

		String buildZipName(String vorgangId) {
			return vorgangController.getVorgang(vorgangId).getNummer() + "_Anhaenge.zip";
		}

		void createZipFile(OutputStream out, String eingangId) {
			var ozgFiles = fileService.getAttachmentsByEingang(eingangId);
			zipDownloadService.write(ozgFiles.toList(), out);
		}
	}

}