package de.ozgcloud.alfa.bescheid;

import static de.ozgcloud.alfa.common.ValidationMessageCodes.*;

import java.util.List;

import jakarta.validation.constraints.NotEmpty;
import jakarta.validation.constraints.NotNull;

import com.fasterxml.jackson.annotation.JsonIgnore;

import de.ozgcloud.alfa.common.LinkedResource;
import de.ozgcloud.alfa.common.ValidationMessageCodes;
import de.ozgcloud.alfa.common.binaryfile.BinaryFileController;
import de.ozgcloud.alfa.common.binaryfile.FileId;
import de.ozgcloud.alfa.common.command.CommandBody;
import lombok.AccessLevel;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.ToString;

@Builder(toBuilder = true)
@Getter
@NoArgsConstructor(access = AccessLevel.PRIVATE)
@AllArgsConstructor(access = AccessLevel.PRIVATE)
@ToString
public class Bescheid implements CommandBody {

	static final String ID_FIELD = "id";
	static final String VERSION_FIELD = "version";
	static final String VORGANG_ID_FIELD = "vorgangId";
	static final String BESCHIEDEN_AM_FIELD = "beschiedenAm";
	static final String BEWILLIGT_FIELD = "bewilligt";
	static final String BESCHEID_DOCUMENT_FIELD = "bescheidDocument";
	static final String ATTACHMENTS_FIELD = "attachments";
	static final String NACHRICHT_TEXT_FIELD = "nachrichtText";
	static final String NARCHRICHT_SUBJECT_FIELD = "nachrichtSubject";
	static final String SEND_BY_FIELD = "sendBy";
	static final String STATUS_FIELD = "status";

	@JsonIgnore
	private String id;
	@JsonIgnore
	private long version;
	@JsonIgnore
	private String vorgangId;

	@NotNull(message = ValidationMessageCodes.FIELD_DATE_FORMAT_INVALID)
	private String beschiedenAm;
	private Boolean bewilligt;

	@LinkedResource(controllerClass = DocumentController.class)
	private String bescheidDocument;

	@LinkedResource(controllerClass = BinaryFileController.class)
	private List<FileId> attachments;
	@NotEmpty(message = FIELD_IS_EMPTY, groups = BescheidNachrichtValidation.class)
	private String nachrichtText;
	@NotEmpty(message = FIELD_IS_EMPTY, groups = BescheidNachrichtValidation.class)
	private String nachrichtSubject;
	private SendBy sendBy;
	private BescheidStatus status;

	private SentInfo sentInfo;
}