package de.ozgcloud.alfa.bescheid;

import java.util.List;
import java.util.function.Predicate;

import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import de.ozgcloud.alfa.common.attacheditem.VorgangAttachedItem;
import de.ozgcloud.alfa.common.command.Command;
import de.ozgcloud.alfa.common.command.CommandOrder;
import de.ozgcloud.alfa.historie.HistorieProcessor;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
@Component
class DocumentHistorieProcessor implements HistorieProcessor {

	static final String DOCUMENT_ITEM_NAME = "Document";

	private static final Predicate<Command> IS_NOT_DOCUMENT_COMMAND = command -> command.getCommandOrder() != CommandOrder.CREATE_BESCHEID_DOCUMENT
			&& command.getCommandOrder() != CommandOrder.CREATE_BESCHEID_DOCUMENT_FROM_FILE;

	private static final Predicate<Command> IS_DOCUMENT_ATTACHED_ITEM = command -> DOCUMENT_ITEM_NAME
			.equals(MapUtils.getString(command.getBody(), VorgangAttachedItem.FIELD_ITEM_NAME, StringUtils.EMPTY));
	private static final Predicate<Command> IS_NOT_CREATE_ATTACHED_ITEM = command -> !(command.getCommandOrder() == CommandOrder.CREATE_ATTACHED_ITEM
			&& IS_DOCUMENT_ATTACHED_ITEM.test(command));

	@Override
	public List<Command> process(List<Command> commands) {
		return filterIrrelevantCommands(commands);
	}

	List<Command> filterIrrelevantCommands(List<Command> commands) {
		return commands.stream().filter(IS_NOT_DOCUMENT_COMMAND).filter(IS_NOT_CREATE_ATTACHED_ITEM).toList();
	}
}
