package de.ozgcloud.alfa.collaboration;

import java.util.Optional;

import org.springframework.hateoas.CollectionModel;
import org.springframework.hateoas.EntityModel;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import lombok.RequiredArgsConstructor;

@RestController
@RequestMapping(OrganisationsEinheitController.PATH)
@RequiredArgsConstructor
class OrganisationsEinheitController {

	static final String PATH = "/api/organisationseinheits"; // NOSONAR
	static final String SEARCH_BY_PARAM = "searchBy";

	private final OrganisationsEinheitService service;
	private final OrganisationsEinheitModelAssembler assembler;
	private final OrganisationsEinheitHeaderModelAssembler headerModelAssembler;

	@GetMapping("/{organisationsEinheitId}")
	public ResponseEntity<EntityModel<OrganisationsEinheit>> getById(@PathVariable String organisationsEinheitId) {
		return ResponseEntity.of(Optional.of(service.getById(organisationsEinheitId)).map(assembler::toModel));
	}

	@GetMapping(params = { SEARCH_BY_PARAM })
	public CollectionModel<EntityModel<OrganisationsEinheitHeader>> search(@RequestParam String searchBy) {
		return headerModelAssembler.toCollectionModel(service.searchOrganisationsEinheiten(searchBy).toList());
	}
}
