package de.ozgcloud.alfa.historie;

import java.util.Optional;

import org.apache.commons.lang3.StringUtils;

import de.ozgcloud.alfa.common.command.Command;
import de.ozgcloud.alfa.common.command.CommandOrder;
import lombok.AccessLevel;
import lombok.NoArgsConstructor;

@NoArgsConstructor(access = AccessLevel.PRIVATE)
class AktenzeichenChangeHistoryBuilder extends ChangeHistoryBuilder<AktenzeichenChangeHistoryBuilder> {

	static final String BODY_PROPERTY_AKTENZEICHEN = "aktenzeichen";

	public static AktenzeichenChangeHistoryBuilder builder() {
		return new AktenzeichenChangeHistoryBuilder();
	}

	@Override
	boolean isRelevant(Command command) {
		return command.getCommandOrder().equals(CommandOrder.SET_AKTENZEICHEN);
	}

	@Override
	CommandWithChangeValues toCommandWithChangeValues(CommandWithPrevious commandWithPrevious) {
		return new CommandWithChangeValues(
				commandWithPrevious.getCommand(),
				getAktenzeichenBeforeChange(commandWithPrevious),
				getAktenzeichenAfterChange(commandWithPrevious));
	}

	String getAktenzeichenBeforeChange(CommandWithPrevious commandWithPrevious) {
		return Optional.ofNullable(commandWithPrevious.getPrevious())
				.map(this::getAktenzeichen)
				.orElse(StringUtils.EMPTY);
	}

	String getAktenzeichenAfterChange(CommandWithPrevious commandWithPrevious) {
		return getAktenzeichen(commandWithPrevious.getCommand());
	}

	String getAktenzeichen(Command command) {
		return getValueFromCommandBody(BODY_PROPERTY_AKTENZEICHEN, command);
	}
}
