package de.ozgcloud.alfa.postfach;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import de.ozgcloud.alfa.common.command.CommandBody;
import de.ozgcloud.alfa.vorgang.VorgangWithEingang;

@Component
class PostfachNachrichtHelper {

	static final String SERVICEKONTO_TYPE_FIELD = "serviceKontoType";

	@Autowired
	private PostfachMailMapper mapper;

	public CommandBody addPostfachAddressToBody(PostfachMail nachricht, VorgangWithEingang vorgang) {
		return addPostfachAddress(nachricht, createPostfachAddressForPostfachMail(vorgang));
	}

	private PostfachMail addPostfachAddress(PostfachMail nachricht, Map<String, Object> postfachAddress) {
		return nachricht.toBuilder().postfachAddress(postfachAddress).build();
	}

	private Map<String, Object> createPostfachAddressForPostfachMail(VorgangWithEingang vorgang) {
		Map<String, Object> postfachAddressForPostfachMail = new HashMap<>(mapper.mapPostfachAddressToMap(getPostfachAddress(vorgang)));
		postfachAddressForPostfachMail.put(SERVICEKONTO_TYPE_FIELD, getServiceKontoType(vorgang));
		return Collections.unmodifiableMap(postfachAddressForPostfachMail);
	}

	private PostfachAddress getPostfachAddress(VorgangWithEingang vorgang) {
		return vorgang.getHeader().getServiceKonto().getPostfachAddresses().get(0);
	}

	private String getServiceKontoType(VorgangWithEingang vorgang) {
		return vorgang.getHeader().getServiceKonto().getType();
	}

}