package de.ozgcloud.alfa.collaboration;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.UUID;
import java.util.stream.Stream;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;

class CollaborationServiceTest {

	@InjectMocks
	private CollaborationService service;

	@Mock
	private CollaborationRemoteService remoteService;

	@Nested
	class TestGetCollaborations {

		private final String id = UUID.randomUUID().toString();

		@Test
		void shouldCallRemoteService() {
			callService();

			verify(remoteService).getCollaborations(id);
		}

		@Test
		void shouldReturnCollaboration() {
			var collaboration = CollaborationTestFactory.create();
			when(remoteService.getCollaborations(id)).thenReturn(Stream.of(collaboration));

			var returnedCollaboration = callService();

			assertThat(returnedCollaboration).contains(collaboration);
		}

		private Stream<Collaboration> callService() {
			return service.getCollaborations(id);
		}
	}

	@Nested
	class TestHasCollaboration {

		private final String id = UUID.randomUUID().toString();

		@Test
		void shouldCallRemoteService() {
			callService();

			verify(remoteService).getCollaborations(id);
		}

		@Test
		void shouldReturnTrue() {
			var collaboration = CollaborationTestFactory.create();
			when(remoteService.getCollaborations(id)).thenReturn(Stream.of(collaboration));

			var hasCollaboration = callService();

			assertThat(hasCollaboration).isTrue();
		}

		@Test
		void shouldReturnFalse() {
			when(remoteService.getCollaborations(id)).thenReturn(Stream.empty());

			var hasCollaboration = callService();

			assertThat(hasCollaboration).isFalse();
		}

		private boolean callService() {
			return service.hasCollaboration(id);
		}
	}
}
