package de.ozgcloud.alfa.common.binaryfile;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.List;
import java.util.Map;

import jakarta.validation.ConstraintValidatorContext;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import com.thedeanda.lorem.LoremIpsum;

class UploadBinaryFileContentTypeValidatorTest {

	@Mock
	private BinaryFileProperties properties;
	@Spy
	@InjectMocks
	private UploadBinaryFileContentTypeValidator validator;

	@Nested
	class TestIsValid {

		@Mock
		private ConstraintValidatorContext context;

		@Test
		void shouldGetConfiguredContentTypes() {
			callValidator();

			verify(validator).getConfiguredContentTypes(UploadBinaryFileTestFactory.FIELD);
		}

		@Test
		void shouldReturnTrueIfConfigurationIsMissing() {
			var result = callValidator();

			assertThat(result).isTrue();
		}

		@Test
		void shouldReturnFalseIfContentTypeIsNotConfigured() {
			givenConfiguredContentTypes(List.of("dummy/abc", "dummy/xyz"));

			var result = callValidator();

			assertThat(result).isFalse();
		}

		@Test
		void shouldReturnTrueIfContentTypeIsConfigured() {
			givenConfiguredContentTypes(List.of(BinaryFileTestFactory.CONTENT_TYPE));

			var result = callValidator();

			assertThat(result).isTrue();
		}

		private void givenConfiguredContentTypes(List<String> contentTypes) {
			doReturn(contentTypes).when(validator).getConfiguredContentTypes(UploadBinaryFileTestFactory.FIELD);
		}

		private boolean callValidator() {
			return validator.isValid(UploadBinaryFileTestFactory.create(), context);
		}
	}

	@Nested
	class TestGetConfiguredContentTypes {

		@Test
		void shouldReturnEmptyCollection() {
			var contentTypes = validator.getConfiguredContentTypes(UploadBinaryFileTestFactory.FIELD);

			assertThat(contentTypes).isEmpty();
		}

		@Test
		void shouldReturnContentTypes() {
			var expectedContentTypes = List.of(LoremIpsum.getInstance().getWords(1), LoremIpsum.getInstance().getWords(1));
			givenConfiguredContentTypes(expectedContentTypes);

			var contentTypes = validator.getConfiguredContentTypes(UploadBinaryFileTestFactory.FIELD);

			assertThat(contentTypes).containsExactlyInAnyOrderElementsOf(expectedContentTypes);
		}

		private void givenConfiguredContentTypes(List<String> contentTypes) {
			when(properties.getContentTypes()).thenReturn(Map.of(UploadBinaryFileTestFactory.FIELD, contentTypes));
		}
	}
}
