/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.errorhandling;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Map;
import java.util.UUID;

import jakarta.validation.ConstraintViolation;
import jakarta.validation.Path;
import jakarta.validation.metadata.ConstraintDescriptor;

import org.hibernate.validator.engine.HibernateConstraintViolation;

import com.thedeanda.lorem.LoremIpsum;

import de.ozgcloud.alfa.common.binaryfile.DynamicViolationParameter;

public class ExceptionTestFactory {

	static final String EXCEPTION_ID = UUID.randomUUID().toString();

	static final String PARAM_NAME = "param";
	static final String PARAM_VALUE = "3";
	static final String PARAM_DYNAMIC_VALUE = "20";
	static final String MESSAGE = LoremIpsum.getInstance().getWords(5);
	static final String MESSAGE_CODE = "message.code";
	private static final String PATH_PREFIX = "createCommandByRelation";
	static final String PATH_FIELD = "command.wiedervorlage.betreff";
	public static final String PATH = PATH_PREFIX + "." + PATH_FIELD;

	public static ConstraintViolation<?> buildMockedConstraintViolation() {
		return ExceptionTestFactory.buildMockedConstraintViolationWithDynamicPayload(null);
	}

	@SuppressWarnings({ "unchecked" })
	public static <T> ConstraintViolation<T> buildMockedConstraintViolationWithDynamicPayload(DynamicViolationParameter dynamicViolationParameter) {
		var violation = mock(ConstraintViolation.class);
		var hibernateViolation = mock(HibernateConstraintViolation.class);
		var constraintDescriptor = mock(ConstraintDescriptor.class);

		var path = mock(Path.class);
		when(path.toString()).thenReturn(PATH);
		when(violation.getPropertyPath()).thenReturn(path);
		when(violation.getMessage()).thenReturn(MESSAGE);
		when(violation.getConstraintDescriptor()).thenReturn(constraintDescriptor);
		when(violation.getInvalidValue()).thenReturn(PARAM_VALUE);
		when(constraintDescriptor.getAttributes()).thenReturn(Map.of(PARAM_NAME, PARAM_DYNAMIC_VALUE));
		when(violation.unwrap(any())).thenReturn(hibernateViolation);
		when(hibernateViolation.getDynamicPayload(any())).thenReturn(dynamicViolationParameter);

		return violation;
	}
}