package de.ozgcloud.alfa.hint;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.Link;

import de.ozgcloud.alfa.RootTestFactory;

class HintRootResourceProcessorTest {

	@InjectMocks
	private HintRootResourceProcessor processor;
	@Mock
	private HintService hintService;

	@DisplayName("HintsLink")
	@Nested
	class TestHintsLink {

		@Test
		void shouldCallHintService() {
			processor.process(EntityModel.of(RootTestFactory.create()));

			verify(hintService).hasHints();
		}

		@Test
		void shouldBePresentIfHintsExists() {
			when(hintService.hasHints()).thenReturn(true);

			var model = processor.process(EntityModel.of(RootTestFactory.create()));

			assertThat(model.getLink(HintRootResourceProcessor.HINTS_RELATION)).get().extracting(Link::getHref).isEqualTo(HintController.PATH);
		}

		@Test
		void shouldNotBePresentIfHintsExists() {
			when(hintService.hasHints()).thenReturn(false);

			var model = processor.process(EntityModel.of(RootTestFactory.create()));

			assertThat(model.getLink(HintRootResourceProcessor.HINTS_RELATION)).isNotPresent();
		}
	}
}