/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.kommentar;

import java.time.ZonedDateTime;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import com.thedeanda.lorem.Lorem;
import com.thedeanda.lorem.LoremIpsum;

import de.ozgcloud.alfa.common.AlfaTestUtils;
import de.ozgcloud.alfa.common.binaryfile.BinaryFileTestFactory;
import de.ozgcloud.alfa.common.binaryfile.FileId;
import de.ozgcloud.alfa.common.command.CreateCommand;
import de.ozgcloud.alfa.common.user.UserProfileTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import de.ozgcloud.common.test.TestUtils;

public class KommentarTestFactory {

	private static Lorem lorem = LoremIpsum.getInstance();

	public static final String ID = AlfaTestUtils.createMongoDbObjectId();
	public static final long VERSION = 73;

	public static final String CREATED_BY = UserProfileTestFactory.ID.toString();
	public static final String CREATED_AT_STR = "2021-01-10T10:30:00Z";
	public static final ZonedDateTime CREATED_AT = ZonedDateTime.parse(CREATED_AT_STR);

	public static final String TEXT = lorem.getWords(15);

	public static final List<FileId> ATTACHMENTS = Collections.singletonList(BinaryFileTestFactory.FILE_ID);

	public static Kommentar create() {
		return createBuilder().build();
	}

	public static Kommentar.KommentarBuilder createBuilder() {
		return Kommentar.builder()
				.id(ID)
				.vorgangId(VorgangHeaderTestFactory.ID)
				.version(VERSION)
				.text(TEXT)
				.attachments(ATTACHMENTS)
				.createdBy(CREATED_BY)
				.createdAt(CREATED_AT);
	}

	public static Map<String, Object> createAsMap() {
		return Map.of(
				KommentarMapper.ID, ID,
				KommentarMapper.TEXT, TEXT,
				KommentarMapper.CREATED_BY, CREATED_BY,
				KommentarMapper.CREATED_AT, CREATED_AT_STR,
				KommentarMapper.ATTACHMENTS, ATTACHMENTS);
	}

	public static String createRequestContent(CreateCommand command) {
		var kommentar = (Kommentar) command.getBody();
		return TestUtils.loadTextFile("jsonTemplates/command/createCommandWithKommentar.json.tmpl",
				command.getOrder(),
				TestUtils.addQuote(kommentar.getText()));
	}
}