package de.ozgcloud.alfa.bescheid;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Optional;

import org.apache.commons.lang3.StringUtils;

import de.ozgcloud.alfa.common.AnlageDokumentTypeBuilder;
import de.ozgcloud.alfa.common.DateConverter;
import de.ozgcloud.alfa.common.IdentifikationObjektTypeBuilder;
import de.ozgcloud.alfa.common.file.OzgFile;
import de.xoev.xdomea.AllgemeineMetadatenType;
import de.xoev.xdomea.AnlageDokumentType;
import de.xoev.xdomea.DokumentType;
import de.xoev.xdomea.HistorienProtokollInformationType;

class DokumentTypeBuilder {
	private static final String TYP = "Bescheid";
	private static final Map<Boolean, String> BEMERKUNG = Map.of(true, "Bescheid wurde genehmigt.", false, "Bescheid wurde abgelehnt.");
	private static final Map<SendBy, String> AKTION = Map.of(SendBy.MANUAL, "Bescheid gespeichert", SendBy.NACHRICHT,
			"Bescheid an Antragsteller gesendet");

	private Bescheid bescheid;
	private String organisationsEinheitenId;
	private String fullName;
	private List<OzgFile> files;

	public static DokumentTypeBuilder builder() {
		return new DokumentTypeBuilder();
	}

	public DokumentTypeBuilder withBescheid(Bescheid bescheid) {
		this.bescheid = bescheid;
		return this;
	}

	public DokumentTypeBuilder withOrganisationseinheitenId(String organisationsEinheitenId) {
		this.organisationsEinheitenId = organisationsEinheitenId;
		return this;
	}

	public DokumentTypeBuilder withFullName(String fullName) {
		this.fullName = fullName;
		return this;
	}

	public DokumentTypeBuilder withFiles(List<OzgFile> files) {
		this.files = files;
		return this;
	}

	public DokumentType build() {
		var dokumentType = new DokumentType();
		dokumentType.setIdentifikation(IdentifikationObjektTypeBuilder.builder().withObjectID(bescheid.getId()).build());
		dokumentType.setTyp(TYP);
		dokumentType.setBezug(StringUtils.defaultString(bescheid.getNachrichtSubject()));
		dokumentType.setAllgemeineMetadaten(createAllgemeineMetadaten());
		dokumentType.getHistorienProtokollInformation().add(createHistorienProtokollInformation());
		Optional.ofNullable(files).orElseGet(Collections::emptyList).stream().map(this::createAnlage).forEach(dokumentType.getAnlage()::add);
		return dokumentType;
	}

	AllgemeineMetadatenType createAllgemeineMetadaten() {
		var metadaten = new AllgemeineMetadatenType();
		metadaten.setBemerkung(BEMERKUNG.get(bescheid.getBewilligt()));
		return metadaten;
	}

	HistorienProtokollInformationType createHistorienProtokollInformation() {
		var protokollInfo = new HistorienProtokollInformationType();
		protokollInfo.setMetadatumName(StringUtils.defaultString(bescheid.getNachrichtText()));
		protokollInfo.setAkteur(fullName + "; " + organisationsEinheitenId);
		protokollInfo.setDatumUhrzeit(DateConverter.toXmlGregorianCalendar(bescheid.getSentInfo().getSentAt()));
		protokollInfo.setAktion(AKTION.get(bescheid.getSendBy()));
		return protokollInfo;
	}

	AnlageDokumentType createAnlage(OzgFile ozgFile) {
		return AnlageDokumentTypeBuilder.builder().withOzgFile(ozgFile).build();
	}
}
