package de.ozgcloud.alfa.export;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.validation.Errors;
import org.springframework.validation.Validator;

import de.ozgcloud.common.errorhandling.TechnicalException;

public class XdomeaPropertiesValidator implements Validator {

	@Override
	public boolean supports(Class<?> clazz) {
		return XdomeaProperties.class.isAssignableFrom(clazz);
	}

	@Override
	public void validate(Object target, Errors errors) {
		var properties = (XdomeaProperties) target;
		validateBehoerdenschluesselProperties(errors, properties);
	}

	private void validateBehoerdenschluesselProperties(Errors errors, XdomeaProperties properties) {
		if (StringUtils.isNotBlank(properties.getBehoerdenschluessel())) {
			validateNotBlank("behoerdenschluesselUri", properties, errors);
			validateNotBlank("behoerdenschluesselVersion", properties, errors);
		}
	}

	private void validateNotBlank(String propertyName, XdomeaProperties properties, Errors errors) {
		getPropertyValue(propertyName, properties);
		if (StringUtils.isBlank(getPropertyValue(propertyName, properties))) {
			errors.rejectValue(propertyName, String.format("ozgcloud.xdomea.%s.empty", propertyName), String.format("%s must be set", propertyName));
		}
	}

	private static String getPropertyValue(String fieldName, XdomeaProperties properties) {
		try {
			return BeanUtils.getSimpleProperty(properties, fieldName);
		} catch (Exception e) {
			throw new TechnicalException("Property does not exist", e);
		}
	}
}
