package de.ozgcloud.alfa.kommentar;

import java.util.List;
import java.util.stream.Stream;

import org.springframework.stereotype.Service;

import de.ozgcloud.alfa.common.binaryfile.BinaryFileService;
import de.ozgcloud.alfa.common.file.OzgFile;
import de.ozgcloud.alfa.common.user.UserId;
import de.ozgcloud.alfa.common.user.UserService;
import de.ozgcloud.alfa.kommentar.KommentarsExportData.KommentarsExportDataBuilder;
import de.ozgcloud.alfa.vorgang.VorgangWithEingang;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
@Service
public class ExportKommentarService {

	private final KommentarService kommentarService;
	private final BinaryFileService binaryFileService;
	private final UserService userService;

	public KommentarsExportData createExportData(VorgangWithEingang vorgang) {
		KommentarsExportDataBuilder builder = KommentarsExportData.builder();
		getKommentare(vorgang)
				.map(kommentar -> createKommentarExportData(vorgang, kommentar))
				.forEach(kommentarExportData -> addKommentarExportData(kommentarExportData, builder));
		return builder.build();
	}

	KommentarExportData createKommentarExportData(VorgangWithEingang vorgang, Kommentar kommentar){
		return new KommentarExportData(vorgang.getOrganisationseinheitenID(), kommentar);
	}

	void addKommentarExportData(KommentarExportData kommentarExportData, KommentarsExportDataBuilder builder) {
		List<OzgFile> attachments = getAttachments(kommentarExportData.kommentar);

		var dokumentType = DokumentTypeBuilder.builder()
				.withKommentarAttachments(attachments)
				.withKommentar(kommentarExportData.kommentar)
				.withOrganisationseinheitenID(kommentarExportData.organisationsEinheitenID)
				.withAuthorFullName(getAuthorFullName(kommentarExportData.kommentar))
				.build();

		builder.dokumentType(dokumentType);
		builder.attachments(attachments);
	}

	List<OzgFile> getAttachments(Kommentar kommentar) {
		return binaryFileService.getFiles(kommentar.getAttachments()).toList();
	}

	String getAuthorFullName(Kommentar kommentar) {
		return userService.getById(UserId.from(kommentar.getCreatedBy())).getFullName();
	}

	Stream<Kommentar> getKommentare(VorgangWithEingang vorgang) {
		return kommentarService.findByVorgangId(vorgang.getId());
	}

	record KommentarExportData(
			String organisationsEinheitenID,
			Kommentar kommentar
	) {
	}
}
